<?php
namespace  Belgian_Tax\Parameter;
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* $Revision$ */

// Copyright (c) 2002 Author Dany De Bontridder dany@alchimerys.be

/*!\file
 * \brief
 */


/*!\brief
 *
 *
 */
#[\AllowDynamicProperties]
class  Tva_Parameter
{
    private static $variable = array("code" => "pcode", "value" => "pvalue", "account" => 'paccount');

    const   SQL_DETAIL_ROW='	select pi_id
	         , pc.tva_id
             ,tva_label
             ,tva_comment
             ,tva_rate
             , pcm_val
             ,tva_code
             ,jrn_def_name
             ,p_side,p_operation
	         , filter_ledger
            from 
                tva_belge.parameter_chld as pc
			left join 
                    tva_rate as tv on (pc.tva_id=tv.tva_id)
			left join
                    jrn_def on (jrn_def_id=ledger_id) ';

    function __construct($p_init)
    {
        $this->cn = $p_init;
    }

    public function get_parameter($p_string)
    {
        if (array_key_exists($p_string, self::$variable)) {
            $idx = self::$variable[$p_string];
            return $this->$idx;
        } else
            throw new \Exception("Attribut inexistant $p_string");
    }

    public function set_parameter($p_string, $p_value)
    {
        if (array_key_exists($p_string, self::$variable)) {
            $idx = self::$variable[$p_string];
            $this->$idx = $p_value;
        } else
            throw new \Exception("Attribut inexistant $p_string");


    }

    public function get_info()
    {
        return var_export(self::$variable, true);
    }

    public function verify()
    {
        // Verify that the elt we want to add is correct
    }

    /**
     * @brief retrieve * row thanks a condition
     */
    public function seek($cond, $p_array = null)
    {
        /*
          $sql="select * from * where $cond";
          return $this->cn->get_array($cond,$p_array)
       */
    }

    public function insert()
    {
        if ($this->verify() != 0) return;

        $sql = "insert into tva_belge.parameter (pcode,pvalue,paccount) " .
            " values ($1,$2) ";
        $this->tva_id = $this->cn->exec_sql(
            $sql,
            array($this->pcode,
                $this->pvalue,
                $this->paccount)
        );

    }

    public function update()
    {
        if ($this->verify() != 0) return;

        $sql = "update tva_belge.parameter  set pvalue=$1,paccount=$2 " .
            " where pcode = $3";
        $res = $this->cn->exec_sql(
            $sql,
            array($this->pvalue,
                $this->paccount,
                $this->pcode)
        );

    }

    public function load()
    {

        $sql = "select pcode from tva_belge.parameter where pcode=$1";

        $res = $this->cn->get_array(
            $sql,
            array($this->pcode)
        );

        if (count($res) == 0) return -1;
        for ($i = 0; $i < count($res); $i++) {
            $this->pcode = $res[$i]['pcode'];
            $this->pvalue = $res[$i]['pvalue'];
            $this->paccount = $res[$i]['paccount'];
        }
        return 0;
    }

    /**
     * @brief check that this accounting does exist in the accounting plan
     * @param $p_code \Belgian_Tax\code to check
     * @return \Belgian_Tax\the number of row found (0 if none)
     */
    public function exist_pcmn($p_code)
    {
        $count = $this->cn->get_value('select count(*) from tmp_pcmn where pcm_val::text like $1', array($p_code));
        return $count;
    }

    /**
     * @brief show the content of the table tva_belge.parameter
     */
    public function display()
    {
        global $cn, $g_tva_parameter;
        $cn = $this->cn;

        $res = $this->cn->get_array("select pcode from tva_belge.parameter");

        ob_start();
        require_once $g_tva_parameter['template'] . '/tva_parameter-display.php';
        $r = ob_get_contents();
        ob_end_clean();
        return $r;
    }

    /**
     * @brief check that TVA_ID exists
     * @param int $tva_id (tva_id.tva_rate)
     * @return int 0 = not exist , 1 exist , 2 returns
     * @throws \Exception DB SQL Exception
     */
    static public function check_tva(int $tva_id): int
    {
        if ( $tva_id == 0) return 2;
        $cn = \Dossier::connect();

        $result = $cn->get_value('select count(*) from tva_rate where tva_id=$1', [$tva_id]);
        return $result;
    }

    /**
     * @brief display the contains of $data (from SQL:TVA_RATE)
     * @param array $data
     * @param int $tva_id tva_id not found , too highlight (0 if nothing to highligt)
     * @return int 0 : success 1 : error
     */
    public static function display_tva(array $data,int $tva_id): int
    {
        $nb_data=count($data);
        if ($nb_data== 0) {
            echo_warning(_('code tva vide !'));
            return 1;
        }
        $both=[0=>'Non',1=>'Oui'];
        echo '<table class="result">';
        echo '<tr>';
        echo th("id");
        echo th("Code");
        echo th("label");
        echo th("taux");
        echo th("description");
        echo th("TVA Achat");
        echo th("TVA Vente");
        echo th("autoliquidataion");
        echo th("exigible achat");
        echo th("exigible vente");
        echo '<tr>';
        for ($z=0;$z<$nb_data;$z++) {
            $class=($z%2==0)?'class="even" ':'class="odd"';
            if ( $data[$z]['tva_id'] == $tva_id) $class=' class="bg-warning"';
            printf ( '<tr %s>',$class);
            echo td($data[$z]["tva_id"]);
            echo td($data[$z]["tva_code"]);
            echo td($data[$z]["tva_label"]);
            echo td($data[$z]["tva_rate"]);
            echo td($data[$z]["tva_comment"]);
            $acc=explode(',',$data[$z]['tva_poste']);
            echo td($acc[0]);
            echo td($acc[1]);
            echo td($both[$data[$z]["tva_both_side"]]);
            echo td($data[$z]["tva_payment_sale"]);
            echo td($data[$z]["tva_payment_purchase"]);
            echo '</tr>';

        }
        echo '</table>';
        return 0;
    }

    /**
     * @brief content of a form to enter information for Parameter Detail (tva_belge.parameter_chld)
     * @param $pcode string Grid (format GRILxx)
     * @param $tab  string open tab
     * @param $pi_id    number PK of table TVA_BELGE.PARAMETER_CHLD , -1 for empty
     * @return string HTML
     * @throws \Exception
     */
    static function input_form($pcode, $tab,$pi_id)
    {
        //variable: $g_user Noalyss_User connected user
        global $g_user;

        // variable: $cn Database
        $cn=\Dossier::connect();

        //variable: $parameter_child load from database
        $parameter_child=new \Belgian_Tax\DB\Parameter_Chld_SQL($cn,$pi_id);

        //variable: $text TVA input
        $text = new \ITva_Popup('tva_id');
        $text->value=$parameter_child->getp('tva_id');
        $text->add_label('tva_label');
        $text->js = 'onchange="this.gDossier=\''.\Dossier::id().'\';set_tva_label(this);"';
        $text->with_button(true);


        // variable: $iposte Accounting
        $iposte=new \IPoste('paccount');
        $iposte->value=$parameter_child->getp('pcm_val');
        $iposte->set_attribute('gDossier',\Dossier::id());
        $iposte->set_attribute('jrn',0);
        $iposte->set_attribute('account','paccount');
        $iposte->set_attribute('label','paccount_label');

        //variable: $select_ledger select a ledger (see security)
        $select_ledger=new \ISelect('ledger_id');

        switch ($tab) {
            case 'opout':
            case 'tvadue':
            case 'opin':
            case 'tvaded':
                $a=$g_user->get_ledger('ACH',3);
                $b=$g_user->get_ledger('VEN',3);
                $all_ledger=array_merge($a,$b);
                break;
            case 'lintra':
            case 'assujetti':
                $all_ledger=$g_user->get_ledger('VEN',3);
                break;

            default:
                throw new \Exception ("TVAAJX235 = tab [$tab] inconnu");
        }
        $default="";
        switch ($tab) {
            case 'opout':
            case 'tvadue':
                $default=1;
                break;
            case 'opin':
            case 'tvaded':
                $default=2;
                break;
        }
        $nb_array=count($all_ledger);
        $select_ledger->value=[];
        //$select_ledger->value=[["value"=>-1,"label"=>_("* aucun filtre *")]];
        for ($x=0;$x < $nb_array;$x++) {
            $select_ledger->value[]=['value'=>$all_ledger[$x]['jrn_def_id']
                ,'label'=>sprintf("(%s) %s "
                    ,$all_ledger[$x]['jrn_def_type']
                    ,$all_ledger[$x]['jrn_def_name'])];
        }
        $select_ledger->selected=$parameter_child->getp('ledger_id');
        
        // var $x INT is the filter_ledger from DB
        $x=$parameter_child->getp('filter_ledger');

        // if $x is null (new record), then depending of the tab or the
        // GRID , we set the default value, 
        if ($x == '' && $pcode != 'GRIL61' && $pcode != 'GRIL62') {
            $x=$default;
        } elseif ($x=='' && ($pcode != 'GRIL61' || $pcode != 'GRIL62')) {
            $x=4;
        } elseif ($x != '')
        {
            $default=$x;
        }
      
        $r='';
        $display_tva='';
         if ( in_array($default, [FILTER_LEDGER_FINANCIAL, FILTER_LEDGER_MISC]))
                 $display_tva='style="display:none" ';
        $r.='<div id="info_tva" '.$display_tva.'>';
        $r.=_(" Code TVA ");
        $r.=$text->input();
        $r.='<span id="tva_label" style="display:block"></span>';
        $r.='</div>';
        $r.=" Poste comptable";
        $r.=$iposte->input();
        $r.=span (_('utilisez le % pour inclure les postes qui en dépendent'),'class="text-muted"');

        $r.=\HtmlInput::hidden('pi_id', $pi_id);
        $r.=\HtmlInput::hidden('tab', $tab);

        $r.='<span id="paccount_label" style="display:inline"></span>';
        $r.='<span style="display:block"></span>';
        $r.=_("Filtré par journal");
        
        $filter_ledger=new \ISelect('filter_ledger');
        
        $a_filter_ledger=[FILTER_LEDGER_SALE=>'Journaux de vente'
            , FILTER_LEDGER_PURCHASE=>'Journaux achat'
            , FILTER_LEDGER_ID=>'Journal précis'
            ];
//----------------------------------------------------------------------------
//  directly use a Accounting from  general ledger
//----------------------------------------------------------------------------
        $a_filter_ledger[FILTER_LEDGER_MISC]='Journaux Opérations diverses';
        $a_filter_ledger[FILTER_LEDGER_FINANCIAL]='Journaux Financiers';
        
        $filter_ledger->transform($a_filter_ledger);
        $filter_ledger->javascript=' onchange=TVA_plugin.check_use_tva() ';
        $filter_ledger->selected=$default;

        $r.=$filter_ledger->input();
        if ( $default != 3 ) $select_ledger->style='style="display:none" ';
        $r.=$select_ledger->input();
        
        $r.='<span style="display:block"></span>';
        $operation=new \ISelect('p_operation');
        $operation->value[]=array('value'=>'A','label'=>_('Addition'));
        $operation->value[]=array('value'=>'S','label'=>_('Soustrait'));
        $operation->selected=$parameter_child->getp('p_operation');
        $r.=_('Opération').$operation->input();
        $r.='<span style="display:block"></span>';
           
        $r.='<div id="amount_side_id" '.$display_tva.'>';
        $r.=_("Montant doit être négatif (ex :Note de crédit )");
        $p_side=new \ICheckBox('p_side','N');
        if ( $parameter_child->getp('p_side') == 'N') $p_side->selected=true;
        $r.=$p_side->input();
        $r.='</div>';
        return $r;
    }

    /**
     * @brief display a row ,
     * @param $p_code string Grid
     * @param $plugin_code plugin code
     * @param $a_code array see SQL_DETAIL_ROW
     * @return string
     * @see SQL_DETAIL_ROW
     */
    public static function print_row($p_code,$plugin_code,$a_code):string
    {
        global $tab;
        $dossier=\Dossier::id();
        $aSide=['P'=>'Positif','N'=>'Négatif'];
        $aOperation=['A'=>'Addition','S'=>'Soustrait'];
        $r='';
        $r=sprintf( '<tr id="tr_%s">',$a_code['pi_id']);
        $r.= td($a_code['tva_id']);
        $r.= td($a_code['tva_label']."(".$a_code['tva_code'].")");
        $r.= td($a_code['tva_comment']);
        $r.= td($a_code['tva_rate']);
        $r.= td($aSide[$a_code['p_side']]);
        $r.= td($aOperation[$a_code['p_operation']]);

        $r.= td("Poste comptable :".$a_code['pcm_val']);
        switch ($a_code['filter_ledger']) {
            case FILTER_LEDGER_SALE:
                $r.= td(\hi(_('Tous les journaux vente')));
                break;
            case FILTER_LEDGER_PURCHASE:
                $r.= td(\hi(_('Tous les journaux achat')));
                break;
            case FILTER_LEDGER_MISC:
                $r.= td(\hi(_('Tous les journaux opérations diverses')));
                break;
            case FILTER_LEDGER_FINANCIAL:
                $r.= td(\hi(_('Tous les journaux financiers')));
                break;
            default:
                $r.= td(\hb($a_code['jrn_def_name']));
        }
        $r.= '<td>';
        $r.= \Icon_Action::modify(uniqid(), sprintf("show_addparam('%s','%s','%s','%s','%s')",
            $p_code,$plugin_code,$dossier,$tab,$a_code['pi_id']));
        $fct=sprintf("delete_param('%s')",$a_code['pi_id']);
        $r.= \Icon_Action::trash("",$fct);
        $r.= '</td>';
        $r.= "</tr>";
        return $r;
    }
}
