<?php

namespace Belgian_Tax;

/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Revision$ */

// Copyright (c) 2002 Author Dany De Bontridder dany@alchimerys.be

/*!
 * \file
 * \brief set up the parameters, save to a file or import from a file
 */

/**
 * @var $http : HttpInput obj
 */
$http = new \HttpInput();

$cn = \Dossier::connect();
\Noalyss\Dbg::echo_file(__FILE__);
$sb = $http->request('sb', "string", "");

//-------------------------------------------------------------------------------------------
// Receive a file to import , this file will replace the current configuration
// but the info for Representative and Declarative
//-------------------------------------------------------------------------------------------
if ($sb == "import_tva") {

    // nothing to save
    if ($_FILES['import_tva']['error'] != 0) {
        echo_warning("IT0: Erreur de chargement du fichier");
        $_REQUEST['tab'] = 'confg';
    } else {
        // upload file
        $file_uploaded = tempnam($_ENV['TMP'], 'tvacfg');

        if (move_uploaded_file($_FILES['import_tva']['tmp_name'], $file_uploaded) == false) {
            echo_warning("IT1: Echec téléchargement du fichier");
            $tvap = new Parameter\Tva_Parameter($cn);
            echo \dossier::hidden();
            echo \HtmlInput::extension();
            echo $tvap->display();
            return;
        }
        try {
            $json_data = file_get_contents($file_uploaded);
            $aData = json_decode($json_data, true);
            if ($aData['version'] != '9024') {
                throw new \Exception("IT3: version fichier inconnue");
            }
            $cn->start();
            // clean parameter and child
            $cn->exec_sql("delete from tva_belge.parameter_chld ");
            $cn->exec_sql('delete from tva_belge.parameter');

            // insert parameter
            $nb_parameter = count($aData['parameter']);

            for ($e = 0; $e < $nb_parameter; $e++) {
                $cn->exec_sql('insert into tva_belge.parameter(pcode) values($1) ', [$aData['parameter'][$e]['pcode']]);
            }

            // insert parameter_child
            $nb_parameter = count($aData['parameter_child']);
            for ($e = 0; $e < $nb_parameter; $e++) {
                if (in_array($aData['parameter_child'][$e]['pcode'], ['CRTVA', 'DTTVA', 'ATVA','ASSUJETTI','CLINTRAGD','CLINTRA','CLINTRATR'])) {
                    // insert special code listing and accounting: CRTVA,DTTVA,ATVA
                    $cn->exec_sql("insert into tva_belge.parameter_chld(pcode, tva_id, pcm_val,ledger_id,filter_ledger,p_operation ,p_side) values ($1,$2,$3,$4,$5,$6,$7)",
                            [
                                $aData['parameter_child'][$e]['pcode']
                                , $aData['parameter_child'][$e]['tva_id']
                                , $aData['parameter_child'][$e]['pcm_val']
                                , $aData['parameter_child'][$e]['ledger_id']
                                , $aData['parameter_child'][$e]['filter_ledger']
                                , 'A'
                                , 'P'
                    ]);
                    continue;
                }
                // var $ledger_type : XX, ACH, ... XX means that this ledger not found
                $ledger_type = "XX";
                switch ($aData['parameter_child'][$e]['filter_ledger']) {
                    case FILTER_LEDGER_ID:
                        $ledger_type = $cn->get_value('select jrn_def_type from jrn_def where jrn_def_id = $1', [$aData['parameter_child'][$e]['ledger_id']]);
                        break;
                    case FILTER_LEDGER_SALE:
                        $ledger_type = 'VEN';
                        break;
                    case FILTER_LEDGER_PURCHASE:
                        $ledger_type = 'ACH';
                        break;
                    case FILTER_LEDGER_FINANCIAL:
                        $ledger_type = 'FIN';
                        break;
                    case FILTER_LEDGER_MISC:
                        $ledger_type = 'ODS';
                        break;
                    default:

                        throw new \Exception("TVAP100 : unknow filter_type");
                }

                // FOR LEDGER 
                if ($ledger_type == 'ACH' || $ledger_type == 'VEN') {
                    // check tva_id
                    $tva_id = ($aData['parameter_child'][$e]['tva_id'] != "") ? $aData['parameter_child'][$e]['tva_id'] : 0;
                    $check_tva = \Belgian_Tax\Parameter\Tva_Parameter::check_tva($tva_id);

                    if ($check_tva == 1) {
                        $cn->exec_sql("insert into tva_belge.parameter_chld(pcode, tva_id, pcm_val,ledger_id,filter_ledger,p_operation ,p_side) values ($1,$2,$3,$4,$5,$6,$7)",
                                [
                                    $aData['parameter_child'][$e]['pcode']
                                    , $aData['parameter_child'][$e]['tva_id']
                                    , $aData['parameter_child'][$e]['pcm_val']
                                    , $aData['parameter_child'][$e]['ledger_id']
                                    , $aData['parameter_child'][$e]['filter_ledger']
                                    , $aData['parameter_child'][$e]['p_operation']
                                    , $aData['parameter_child'][$e]['p_side']
                        ]);
                    } else {
                        echo ('<p class="error"> configurer les TVA manquantes dans C0TVA</p>');
                        \Belgian_Tax\Parameter\Tva_Parameter::display_tva(
                                $aData['tva_rate']
                                , $tva_id
                        );
                        throw new \Exception(
                                        "Chargement annulé : aucun changement enregistré " .
                                        "CODE TVA {$aData['parameter_child'][$e]['tva_id']} inconnu"
                                );
                    }
                } elseif ($ledger_type == 'ODS' || $ledger_type == 'FIN') {
                    $cn->exec_sql("insert into tva_belge.parameter_chld(pcode, tva_id, pcm_val,ledger_id,filter_ledger,p_operation ,p_side) values ($1,$2,$3,$4,$5,$6,$7)",
                            [
                                $aData['parameter_child'][$e]['pcode']
                                , $aData['parameter_child'][$e]['tva_id']
                                , $aData['parameter_child'][$e]['pcm_val']
                                , $aData['parameter_child'][$e]['ledger_id']
                                , $aData['parameter_child'][$e]['filter_ledger']
                                , $aData['parameter_child'][$e]['p_operation']
                                , $aData['parameter_child'][$e]['p_side']
                    ]);
                }  else {
                    echo '<p class="bg-warning">';
                    printf('ATTENTION Grille %s journal inconnu %s '
                            , $aData['parameter_child'][$e]['pcode']
                            , $aData['parameter_child'][$e]['ledger_id']);
                          
                    
                    echo '</p>';
                }
            }
            $cn->commit();
        } catch (\Exception $e) {
            $cn->rollback();
            echo_warning($e->getMessage());
        }
        $_REQUEST['tab'] = 'confg';
    }
}
//-------------------------------------------------------------------------------------------
// save all the parameters
//
//-------------------------------------------------------------------------------------------
if (isset($_POST['save_misc'])) {

    foreach (array('CRTVA', 'ATVA', 'DTTVA') as $i) {
        $value = $http->post($i);
        if (noalyss_trim($value) == '')
            $value = null;
        if ($cn->get_value("select count(*) from tva_belge.parameter_chld where pcode=$1", array($i)) == 0) {
            $cn->exec_sql("insert into tva_belge.parameter_chld(pcode,pcm_val) values($1,$2) ",
                    array($i, $value));
        } else {

            $cn->exec_sql("update tva_belge.parameter_chld set pcm_val=$1::account_type where pcode=$2",
                    array($value, $i));
        }
    }
    try {

        $declarant = new \Belgian_Tax\XML\Declarant();
        $declarant->fromPost();
        $declarant->save();
        $representative = new \Belgian_Tax\XML\Representative();
        $representative->fromPost();
        $representative->save();
        unset($_POST['save_misc']);
    } catch (\Exception $e) {
        print h2($e->getMessage(), 'class="error"');
    }
}
//-------------------------------------------------------------------------------------------
// save parameter for TVA
//-------------------------------------------------------------------------------------------
/* show all the possible parameters */
$tvap = new Parameter\Tva_Parameter($cn);

echo \dossier::hidden();
echo \HtmlInput::extension();

echo $tvap->display();

